--------------------------------------------------------------------------------
-- Company:    Digilent inc
-- Engineer:   Kovacs Laszlo - Attila
--
-- Create Date:    12:11:33 03/26/07
-- Design Name:    
-- Module Name:    DA2 - Behavioral
-- Project Name:   
-- Target Device:  X-Board, PModDA2
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DA2 is
    Port ( 
      Clk      : in  std_logic;

      DataIn   : in  std_logic_vector(7 downto 0);
      DataOut  : out std_logic_vector(7 downto 0);
      SelectAB : in  std_logic;
      LowHigh  : in  std_logic;
      Read     : in  std_logic;
      Write    : in  std_logic;
      DataRdy  : out std_logic;

      Sync     : out std_logic;
      DinA     : out std_logic;
      DinB     : out std_logic;
      SClk     : out std_logic);
end DA2;

architecture Behavioral of DA2 is

-- register for the DA2 channel A
signal dataA  : std_logic_vector(15 downto 0) := x"0000";

-- register for the DA2 channel B
signal dataB  : std_logic_vector(15 downto 0) := x"0000";

signal data   : std_logic_vector(15 downto 0) := x"0000";
signal cnt  : std_logic_vector(4 downto 0) := "10000";
signal idle : std_logic;
signal update : std_logic := '0';

begin
   
   -- select the channel A or B, 
   data <= dataA when SelectAB = '0' else dataB;

   DataOut <= data(15 downto 8) when LowHigh = '1' else data(7 downto 0);
   
   -- Sync (CS, SS) signal for the DA2
   Sync <= idle;
   -- data for the channels A and B
   DinA <= dataA(15);
   DinB <= dataB(15);
   -- clock for the DA2
   SClk <= Clk;
   
   -- serializer operation finished
   idle <= '1' when cnt = "10000" else '0';
                                                             
   process(Clk)
   begin
      if rising_edge(Clk) then

         -- on Write of the last (second) confuration byte 
         -- set the flag for update
         -- and clear when the DA2 is configured with it
         if Write = '1' and LowHigh = '1' then
            update <= '1';
         elsif idle = '1' then
            update <= '0';
         end if;

         -- signal for the EPP module that the data is ready:
         --  immediately when a read is performed 
         --   or first byte is loaded 
         --  after the configuration updated is finished 
         if Read = '1' or (Write = '1' and LowHigh = '0') or 
            (update = '1' and idle = '1') then
            
            DataRdy <= '1';
         else
            DataRdy <= '0';
         end if;
         
         -- on Write signal load the data in the propper register
         if Write = '1' then
            if LowHigh = '0' then
               if SelectAB = '0' then
                  dataA(7 downto 0) <= DataIn;
               else
                  dataB(7 downto 0) <= DataIn;
               end if;
            else
               if SelectAB = '0' then
                  dataA(15 downto 8) <= DataIn;               
               else
                  dataB(15 downto 8) <= DataIn;               
               end if;
            end if;
            
         -- while the rotate signale is active 
         -- rotate the configuration registers for the data to be serialized
         elsif idle = '0' then
            dataA <= dataA(14 downto 0)&dataA(15);
            dataB <= dataB(14 downto 0)&dataB(15);
         end if;
         
         -- when the last (second) configuration byte is received 
         -- start the serialization (DA2 configuraion)
         if Write = '1' and LowHigh = '1' then
            cnt <= (others => '0');
         elsif idle = '0' then
            cnt <= cnt + '1';
         end if;

      end if;
   end process;
   

end Behavioral;
